#ifndef DISPLAY_H_INCLUDED
#define DISPLAY_H_INCLUDED

#include "gamestatus.hpp"
#include "key.hpp"
#include "map.hpp"
#include "pathfind.hpp"
#include "team.hpp"
#include "unit.hpp"
#include "video.hpp"

#include "SDL.h"

#include <map>
#include <string>

class display
{
public:
	typedef std::map<gamemap::location,unit> unit_map;
		
	display(const unit_map& units, CVideo& video,
	        const gamemap& map, const gamestatus& status,
			const std::vector<team>& t);
	~display();

	void scroll(double xmov, double ymov);
	void zoom(double amount);

	enum SCROLL_TYPE { SCROLL, WARP };
	void scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type=SCROLL);
	
	void draw(bool update=true);

	int x() const;
	int mapx() const;
	int y() const;

	void select_hex(gamemap::location hex);
	void highlight_hex(gamemap::location hex);
	gamemap::location hex_clicked_on(int x, int y);

	void set_paths(const paths* paths_list);

	void move_unit(const std::vector<gamemap::location>& path, const unit& u);

	CVideo& video() { return screen_; }
	
private:
	display(const display&);
	void operator=(const display&);
	
	void update_display();
	
	void move_unit_between(const gamemap::location& a,
					       const gamemap::location& b,
						   const SDL_Surface* image);
	void draw_moving_unit(int x, int y, const SDL_Surface* image);
	
	void draw_sidebar();
	void draw_tile(int x, int y);
	void draw_minimap(int x, int y, int w, int h);
	void draw_unit_details(int x, int y);
	void draw_game_status(int x, int y);

	void bounds_check_position();

	enum IMAGE_TYPE { UNSCALED, SCALED, GREYED, BRIGHTENED };
	SDL_Surface* getImage(const std::string& filename,IMAGE_TYPE type=SCALED);

	enum TINT { GREY_IMAGE, BRIGHTEN_IMAGE };
	SDL_Surface* getImageTinted(const std::string& filename, TINT tint);
	SDL_Surface* getMinimap(int w, int h);

	void clearImageCache();
	
	CVideo& screen_;
	double xpos_, ypos_, zoom_;
	const gamemap& map_;

	gamemap::location selectedHex_;
	gamemap::location mouseoverHex_;

	const unit_map& units_;

	std::map<std::string,SDL_Surface*> images_, scaledImages_,
	                                   greyedImages_, brightenedImages_;
	SDL_Surface* minimap_;

	const paths* pathsList_;

	const gamestatus& status_;

	const std::vector<team>& teams_;
};

#endif
