#include "map.hpp"

#include <cstdlib>

const char* gamemap::terrain_name(gamemap::TERRAIN terrain)
{
	switch(terrain) {
		case VOID_TERRAIN: return "none";
		case DEEP_WATER: return "ocean";
		case SHALLOW_WATER: return "coast";
		case GRASSLAND: return "grassland";
		case DESERT: return "desert";
		case FOREST: return "forest";
		case HILLS: return "hills";
		case MOUNTAINS: return "mountains";
		case TOWER: return "tower";
		case CASTLE: return "castle";
		default: return "none";
	}
}

bool gamemap::location::operator==(const gamemap::location& a) const
{
	return x == a.x && y == a.y;
}

bool gamemap::location::operator!=(const gamemap::location& a) const
{
	return !operator==(a);
}


bool gamemap::location::operator<(const gamemap::location& a) const
{
	return x < a.x || x == a.x && y < a.y;
}

gamemap::gamemap(const std::string& data) : tiles_(1)
{
	int x = 0, y = 0;
	for(std::string::const_iterator i = data.begin(); i != data.end(); ++i) {
		if(*i == '\n') {
			tiles_.push_back(std::vector<TERRAIN>());
			y = 0;
			++x;
		} else {
			switch(*i) {
				case 's':
					tiles_.back().push_back(DEEP_WATER);
					break;
				case 'c':
					tiles_.back().push_back(SHALLOW_WATER);
					break;
				case 'g':
					tiles_.back().push_back(GRASSLAND);
					break;
				case 'd':
					tiles_.back().push_back(DESERT);
					break;
				case 'f':
					tiles_.back().push_back(FOREST);
					break;
				case 'h':
					tiles_.back().push_back(HILLS);
					break;
				case 'm':
					tiles_.back().push_back(MOUNTAINS);
					break;
				case 't':
					tiles_.back().push_back(TOWER);
					towers_.push_back(location(x,y));
					break;
				case 'C':
					tiles_.back().push_back(CASTLE);
					break;
				default:
					if(!isdigit(*i))
						throw incorrect_format_exception("Illegal character");

					startingPositions_[*i - '0'] = location(x,y);

					tiles_.back().push_back(CASTLE);
			}

			++y;
		}
	}

	if(tiles_.empty())
		throw incorrect_format_exception("empty map");

	for(int n = 0; n != tiles_.size(); ++n) {
		if(tiles_[n].size() != this->y()) {
			tiles_.erase(tiles_.begin()+n);
			--n;
		}
	}
}

int gamemap::x() const { return tiles_.size(); }
int gamemap::y() const { return tiles_[0].size(); }

const std::vector<gamemap::TERRAIN>& gamemap::operator[](int index) const
{
	return tiles_[index];
}

const gamemap::location& gamemap::starting_position(int n) const
{
	return startingPositions_[n];
}
