#ifndef MAP_H_INCLUDED
#define MAP_H_INCLUDED

#include <string>
#include <vector>

class gamemap
{
public:
		
	enum TERRAIN { VOID_TERRAIN, DEEP_WATER, SHALLOW_WATER, GRASSLAND, DESERT,
	               FOREST, HILLS, MOUNTAINS, TOWER, CASTLE };

	static const char* terrain_name(TERRAIN terrain);
				   
	struct incorrect_format_exception {
		incorrect_format_exception(const char* msg) : msg_(msg) {}
		const char* const msg_;
	};

	struct location {
		location() : x(-1), y(-1) {}
		location(int x, int y) : x(x), y(y) {}

		bool valid() const { return x > 0 && y > 0; }

		int x, y;

		bool operator<(const location& a) const;
		bool operator==(const location& a) const;
		bool operator!=(const location& a) const;
	};
				   
	gamemap(const std::string& data); //throw(incorrect_format_exception)

	int x() const;
	int y() const;
	
	const std::vector<TERRAIN>& operator[](int index) const;
	
	const location& starting_position(int n) const;

	bool on_board(const location& loc) const
	{
		return loc.valid() && loc.x < x() && loc.y < y();
	}

	const std::vector<location>& towers() const { return towers_; }
private:
	std::vector<std::vector<TERRAIN> > tiles_;
	std::vector<location> towers_;
	location startingPositions_[10];
};

//bool operator==(const gamemap::location& a, const gamemap::location& b);
//bool operator<(const gamemap::location& a, const gamemap::location& b);

#endif
