#include "menu.hpp"

int show_menu(int x, int y, CVideo& screen,
              const std::vector<std::string>& items)
{
	bool left_button = true, right_button = true;
	for(;;) {
		int mousex, mousey;
		const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);

		const bool new_right_button = mouse_flags&SDL_BUTTON_RMASK;
		if(!right_button && new_right_button)
			return -1;

		right_button = new_right_button;
		
		const bool new_left_button = mouse_flags&SDL_BUTTON_LMASK;

		for(int i = 0; i != items.size(); ++i) {
			const int xloc = x;
			const int yloc = y+i*9;
			const int xdim = items[i].size()*8;
			const int ydim = 8;

			int colour = 0xFFFFFF;
			if(mousex > xloc && mousex < xloc + xdim &&
			   mousey > yloc && mousey < yloc + ydim) {
				colour = 0xFFFFAA;
				if(new_left_button && !left_button) {
					return i;
				}
			}
			
			screen.drawText(x,y+i*9,colour,0x0,items[i].c_str());
		}

		screen.update(0,0,screen.getSurface()->w,screen.getSurface()->h);

		//if someone has clicked outside of the menu area, close the menu
		if(new_left_button && !left_button) {
			return -1;
		}

		left_button = new_left_button;

		SDL_PumpEvents();
	}
}

void show_message(int x, int y, CVideo& screen, const std::string& message)
{
	screen.drawText(x,y,0xFFFFFF,0x0,message.c_str());
	screen.update(0,0,screen.getSurface()->w,screen.getSurface()->h);

	bool left_button = true, right_button = true;
	for(;;) {
		int mousex, mousey;
		const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);

		const bool new_left_button = mouse_flags&SDL_BUTTON_LMASK;
		const bool new_right_button = mouse_flags&SDL_BUTTON_RMASK;
		if(new_left_button && !left_button || new_right_button && !right_button)
			break;

		left_button = new_left_button;
		right_button = new_right_button;
		SDL_PumpEvents();
	}
}
