#include "unit.hpp"

#include <cstdlib>
#include <sstream>

unit::unit(game_data& data, config& cfg) : moves_(0), recruit_(false)
{
	read(data,cfg);
}

unit::unit(const unit_type* t, int side) :
               type_(t), hitpoints_(t->hitpoints()),
			   experience_(0), side_(side), moves_(0),
			   recruit_(false)
{}

const unit_type& unit::type() const
{
	return *type_;
}

const std::string& unit::description() const
{
	return description_;
}

int unit::side() const
{
	return side_;
}

bool unit::can_recruit() const
{
	return recruit_;
}

int unit::movement_left() const
{
	return moves_ < 0 ? 0 : moves_;
}

bool unit::can_attack() const
{
	return moves_ != -1;
}

void unit::set_movement(int moves)
{
	if(moves_ != -1)
		moves_ = moves;
}

void unit::set_attacked()
{
	moves_ = -1;
}

void unit::new_turn()
{
	moves_ = type().movement();
}

int unit::hitpoints() const
{
	return hitpoints_;
}

int unit::experience() const
{
	return experience_;
}

bool unit::gets_hit(int damage)
{
	hitpoints_ -= damage;
	return hitpoints_ <= 0;
}

void unit::read(game_data& data, config& cfg)
{
	std::map<std::string,unit_type>::iterator i = data.unit_types.find(
					                                     cfg.values["type"]);
	if(i != data.unit_types.end())
		type_ = &i->second;
	else
		type_ = 0;

	hitpoints_ = atoi(cfg.values["hitpoints"].c_str());
	experience_ = atoi(cfg.values["experience"].c_str());
	side_ = atoi(cfg.values["side"].c_str());
	description_ = cfg.values["description"];
	const std::map<std::string,std::string>::const_iterator recruit_itor =
			                                 cfg.values.find("canrecruit");
	if(recruit_itor != cfg.values.end() && recruit_itor->second == "1") {
		recruit_ = true;
	}
}

std::string unit::write() const
{
	config cfg;
	cfg.values["type"] = type_->name();

	std::stringstream hp;
	hp << hitpoints_;
	cfg.values["hitpoints"] = hp.str();

	std::stringstream xp;
	xp << experience_;
	cfg.values["experience"] = xp.str();

	std::stringstream sd;
	sd << side_;
	cfg.values["side"] = sd.str();

	cfg.values["description"] = description_;

	return cfg.write();
}
