#include <cassert>
#include <fstream>
#include <iostream>
#include <stack>

#include "config.hpp"

std::string read_file(const std::string& fname)
{
	std::ifstream file(fname.c_str());
	std::string res;
	char c;
	while(file.get(c)) {
		res.resize(res.size()+1);
		res[res.size()-1] = c;
	}

	return res;
}

config::config(const std::string& data)
{
	read(data);
}

config::~config()
{
	cleanup();
}

void config::read(const std::string& data)
{
	std::stack<config*> elements;
	elements.push(this);
	
	enum { INITIAL, ELEMENT_NAME, IN_ELEMENT, VARIABLE_NAME, VALUE }
	  state = INITIAL;
	std::string var;
	std::string value;
	for(std::string::const_iterator i = data.begin(); i != data.end(); ++i) {
		const char c = *i;
		switch(state) {
			case INITIAL:
				if(c == '[')
					state = ELEMENT_NAME;
				
				break;

			case ELEMENT_NAME:
				if(c == ']') {
					if(value == "end") {
						assert(!elements.empty());

						elements.pop();

						if(elements.empty())
							return;

						state = IN_ELEMENT;
						
						break;
					}
					
					config* const new_config = new config();
					elements.top()->children[value].push_back(new_config);
					elements.push(new_config);
					state = IN_ELEMENT;
					value = "";
				} else {
					value.resize(value.size()+1);
					value[value.size()-1] = c;
				}

				break;

			case IN_ELEMENT:
				if(c == '[') {
					state = ELEMENT_NAME;
					value = "";
				} else if(!isspace(c)) {
					value.resize(1);
					value[0] = c;
					state = VARIABLE_NAME;
				}

				break;

			case VARIABLE_NAME:
				if(c == '=') {
					state = VALUE;
					var = value;
					value = "";
				} else {
					value.resize(value.size()+1);
					value[value.size()-1] = c;
				}

				break;

			case VALUE:
				if(c == '\n') {
					state = IN_ELEMENT;
					elements.top()->values.insert(
							std::pair<std::string,std::string>(var,value));
					var = "";
					value = "";
				} else {
					value.resize(value.size()+1);
					value[value.size()-1] = c;
				}

				break;
		}
	}
}

std::string config::write() const
{
	std::string res;
	for(std::map<std::string,std::string>::const_iterator i = values.begin();
					i != values.end(); ++i) {
		res += i->first + "=" + i->second + "\n";
	}

	for(std::map<std::string,std::vector<config*> >::const_iterator j =
					children.begin(); j != children.end(); ++j) {
		const std::vector<config*>& v = j->second;
		for(std::vector<config*>::const_iterator it = v.begin();
						it != v.end(); ++it) {
			res += "[" + j->first + "]\n";
			res += (*it)->write();
			res += "[end]\n";
		}
	}

	res += "\n";

	return res;
}

void config::cleanup()
{
	for(std::map<std::string,std::vector<config*> >::iterator i =
					children.begin(); i != children.end(); ++i) {
		std::vector<config*>& v = i->second;
		for(std::vector<config*>::iterator j = v.begin(); j != v.end(); ++j)
			delete *j;
	}

	children.clear();
	values.clear();
}

//#define TEST_CONFIG

#ifdef TEST_CONFIG

int main()
{
	config cfg(read_file("testconfig"));
	std::cout << cfg.write() << std::endl;
}

#endif
