#ifndef LOG_HPP_INCLUDED
#define LOG_HPP_INCLUDED

#define LOG_DATA

#ifdef LOG_DATA

#include <iostream>
#include <string>

struct scope_logger
{
	scope_logger(const std::string& str) : str_(str) {
		for(int i = 0; i != indent; ++i)
			std::cerr << "  ";
		++indent;
		std::cerr << "BEGIN: " << str_ << "\n";
	}

	~scope_logger() {
		--indent;
		for(int i = 0; i != indent; ++i)
			std::cerr << "  ";
		std::cerr << "END: " << str_ << "\n";
	}

private:
	std::string str_;
	static int indent;
};

int scope_logger::indent = 0;

#define log(a) std::cerr << a << "\n";
#define log1(a,b) std::cerr << a << " info: " << b << "\n";
#define log2(a,b,c) std::cerr << a << " info: " << b << ", " << c << "\n";

#define log_scope(a) scope_logger scope_logging_object__(a);

#else
#define log(a)
#define log1(a,b)
#define log2(a,b,c)

#define log_scope(a)
#endif

#endif
