#include "pathfind.hpp"

#include <cmath>
#include <cstdlib>


void get_adjacent_tiles(const gamemap::location& a, gamemap::location* res)
{
	res->x = a.x;
	res->y = a.y-1;
	++res;
	res->x = a.x+1;
	res->y = a.y + ((a.x%2) == 0 ? -1:0);
	++res;
	res->x = a.x+1;
	res->y = a.y + ((a.x%2) == 0 ? 0:1);
	++res;
	res->x = a.x;
	res->y = a.y+1;
	++res;
	res->x = a.x-1;
	res->y = a.y + ((a.x%2) == 0 ? 0:1);
	++res;
	res->x = a.x-1;
	res->y = a.y + ((a.x%2) == 0 ? -1:0);
}

namespace {

bool tiles_adjacent(const gamemap::location& a, const gamemap::location& b)
{
	//two tiles are adjacent if y is different by 1, and x by 0, or if
	//x is different by 1 and y by 0, or if x and y are each different by 1,
	//and the x value of the hex with the greater y value is odd
	
	const int xdiff = abs(a.x - b.x);
	const int ydiff = abs(a.y - b.y);
	return ydiff == 1 && a.x == b.x || xdiff == 1 && a.y == b.y ||
	   xdiff == 1 && ydiff == 1 && (a.y > b.y ? (a.x%2) == 1 : (b.x%2) == 1);
}

bool enemy_zoc(const std::map<gamemap::location,unit>& units,
               const gamemap::location& loc, int side)
{
	gamemap::location locs[6];
	get_adjacent_tiles(loc,locs);
	for(int i = 0; i != 6; ++i) {
		const std::map<gamemap::location,unit>::const_iterator it
				= units.find(locs[i]);
		if(it != units.end() && it->second.side() != side) {
			return true;
		}
	}

	return false;
}

void find_routes(const gamemap& map, const game_data& gamedata,
				 const std::map<gamemap::location,unit>& units,
				 const unit& u,
				 const gamemap::location& loc,
				 int move_left,
				 std::map<gamemap::location,paths::route>& routes)
{
	//find the adjacent tiles and iterate over them
	gamemap::location locs[6];
	get_adjacent_tiles(loc,locs);
	for(int i = 0; i != 6; ++i) {
		const gamemap::location& currentloc = locs[i];

		//check if the adjacent location is off the board
		if(currentloc.x < 0 || currentloc.y < 0 ||
		   currentloc.x >= map.x() || currentloc.y >= map.y())
			continue;

		//see if the tile is on top of an enemy unit
		const std::map<gamemap::location,unit>::const_iterator unit_it =
				units.find(locs[i]);
		if(unit_it != units.end() && unit_it->second.side() != u.side())
			continue;

		//find the terrain of the adjacent location
		const gamemap::TERRAIN terrain = map[currentloc.x][currentloc.y];

		//find the movement type of the unit
		std::map<std::string,unit_movement_type>::const_iterator it =
				gamedata.movement_types.find(u.type().movement_type());
		if(it != gamedata.movement_types.end()) {

			//find the movement cost of this type onto the terrain
			const int move_cost = it->second.movement_cost(terrain);
			if(move_cost <= move_left) {
				const std::map<gamemap::location,paths::route>::const_iterator
						rtit = routes.find(currentloc);

				//if a better route to that tile has already been found
				if(rtit != routes.end() &&
				   rtit->second.move_left >= move_left - move_cost)
					continue;

				const bool zoc = enemy_zoc(units,currentloc,u.side());
				paths::route new_route = routes[loc];
				new_route.steps.push_back(loc);
				new_route.move_left = zoc ? 0 : move_left - move_cost;
				routes[currentloc] = new_route;

				if(new_route.move_left > 0) {
					find_routes(map,gamedata,units,u,currentloc,
					            new_route.move_left,routes);
				}
			}
		}
	}
}

} //end anon namespace

paths::paths(const gamemap& map, const game_data& gamedata,
             const std::map<gamemap::location,unit>& units,
             const gamemap::location& loc)
{
	const std::map<gamemap::location,unit>::const_iterator i = units.find(loc);
	if(i == units.end())
		return;

	routes[loc].move_left = i->second.movement_left();
	find_routes(map,gamedata,units,i->second,loc,
	            i->second.movement_left(),routes);

	if(i->second.can_attack()) {
		gamemap::location adjacent[6];
		get_adjacent_tiles(loc,adjacent);
		for(int j = 0; j != 6; ++j) {
			const std::map<gamemap::location,unit>::const_iterator enemy =
					units.find(adjacent[j]);
			if(enemy != units.end() &&
			   enemy->second.side() != i->second.side()) {
				route new_route;
				new_route.move_left = -1;
				routes.insert(std::pair<gamemap::location,route>(
										          adjacent[j],new_route));	
			}
		}
	}
}

