#include "game_config.hpp"
#include "team.hpp"

team::team_info::team_info(config& cfg)
{
	name = cfg.values["name"];
	const std::vector<config*>& recruits = cfg.children["recruit"];
	for(std::vector<config*>::const_iterator i = recruits.begin();
					i != recruits.end(); ++i) {
		can_recruit.insert((*i)->values["name"]);
	}
}

team::team(const team::team_info& info) : gold_(100), info_(&info)
{}

void team::get_tower(const gamemap::location& loc)
{
	towers_.insert(loc);
}

void team::lose_tower(const gamemap::location& loc)
{
	towers_.erase(towers_.find(loc));
}

int team::towers() const
{
	return towers_.size();
}

bool team::owns_tower(const gamemap::location& loc) const
{
	return towers_.count(loc) > 0;
}

int team::gold() const
{
	return gold_;
}

void team::new_turn()
{
	gold_ += towers_.size()*game_config::tower_income+game_config::base_income;
}

void team::spend_gold(int amount)
{
	gold_ -= amount;
}

const std::set<std::string>& team::recruits() const
{
	return info_->can_recruit;
}

const std::string& team::name() const
{
	return info_->name;
}
