#ifndef UNIT_TYPES_H_INCLUDED
#define UNIT_TYPES_H_INCLUDED

#include "config.hpp"
#include "map.hpp"
#include "team.hpp"

#include <string>
#include <vector>

//the 'attack type' is the type of attack, how many times it strikes,
//and how much damage it does.
class attack_type
{
public:
	enum RANGE { SHORT_RANGE, LONG_RANGE };
		
	attack_type(config& cfg);
	const std::string& name() const;
	const std::string& type() const;
	RANGE range() const;
	int damage() const;
	int num_attacks() const;
private:
	mutable config* cfg_;
};

//the 'unit movement type' is the basic size of the unit - aerial, small land,
//large land, etc etc.
class unit_movement_type
{
public:
	unit_movement_type(config& cfg);

	const std::string& name() const;
	int movement_cost(gamemap::TERRAIN terrain) const;
	double defense_modifier(gamemap::TERRAIN terrain) const;
	int damage_against(const attack_type& attack) const;

private:
	mutable config& cfg_;
};

class unit_type
{
public:
	unit_type(config& cfg);

	const std::string& name() const;
	const std::string& image() const;
	const std::string& image_profile() const;
	int hitpoints() const;
	std::vector<attack_type> attacks() const;
	const std::string& movement_type() const;
	
	int experience_needed() const;
	const std::string& advances_to() const;
	const std::string& usage() const;

	int level() const;
	int movement() const;
	int cost() const;

	enum ALIGNMENT { LAWFUL, NEUTRAL, CHAOTIC };

	ALIGNMENT alignment() const;
	static const std::string& alignment_description(ALIGNMENT align);

	const std::string& ability() const;

	bool heals() const;
	bool regenerates() const;
private:
	mutable config& cfg_;
};

struct game_data
{
	game_data(config& cfg);
	std::map<std::string,unit_movement_type> movement_types;
	std::map<std::string,unit_type> unit_types;
	std::map<std::string,team::team_info> races;

private:
	config& cfg_;
};

#endif
